package cache

import (
	"sync"
	"time"
)

/*********************************************************全国行政区域**/
var provinceMap []District // 省份列表[]District
var cityMap sync.Map       // 城市列表map[uint32][]District  //省份ID作为KEY
var countyMap sync.Map     // 区县列表map[uint32][]District  //城市ID作为KEY
type District struct {
	Id      uint32 `json:"id" example:"1"`
	Name    string `json:"name" example:"四川省"`
	Initial string `json:"initial" example:"S"`
}

/*********************************************************代理商**/
//// AgentMap 代理商
//var AgentMap sync.Map
/*---------------------------------------------------------------*/

/*********************************************************运营商**/
var operatorMap sync.Map //运营商id作为KEY
type OperatorStruct struct {
	ShortName string
	// 可再添加运营商属性
}

/*---------------------------------------------------------------*/

/******************************************************区域(校区)服务信息**/
var areaServiceMap sync.Map     //area_service_id 作为KEY  AreaStruct 作为VALUE
type AreaServiceStruct struct { //
	AreaID     uint32
	ServiceID  uint8  //服务项目 1公话 2定位 3饮水 4POS .........
	OperatorID uint32 //运营商ID
	Name       string //服务名字(例宿舍135栋饮水)name
	Status     uint8  //状态 0无服务1暂停10正式11测试
}

/*---------------------------------------------------------------*/

/******************************************************后台管理用户**/
var adminMap sync.Map //后台管理用户  LoginAccount(string) 作为KEY
type AdminStruct struct {
	Password       [16]byte
	tokenCreatTime uint32
	Name           string
	upAdminID      uint32
	//Phone string
	//OperatorID []int
	SystemAuth      uint32     //系统角色权限
	AreaServiceAuth [][]uint32 //area_service_id,admin_role_id
}

/*---------------------------------------------------------------*/

// roleAuthMap /****************************角色权限(包含区域和系统角色)*/
var adminRoleAuthMap sync.Map //admin_role_id 作为KEY
type RoleAuthStruct struct {
	Name     string
	MenuRole map[uint32][]uint32 // 菜单、按钮权限
}

/*---------------------------------------------------------------*/

/******************************************************区域(校区)**/
var areaMap sync.Map     //area_id 作为KEY  AreaStruct 作为VALUE
type AreaStruct struct { // AreaStruct 区域信息缓存，通过区域ID查出,后续再加支付信息
	Name     string
	Loc      [2]uint32
	CountyID uint32
	Status   bool //校区状态
}

/*---------------------------------------------------------------*/

/***************************************************微信用户(家长)**/
var wechatCustomerMap sync.Map //微信的WechatUnionID(string)作为KEY
type WechatCustomerStruct struct {
	tokenCreatTime   uint32
	testRole         uint8    // 测试用户(可看到测试校区)
	nickname         string   //微信昵称
	masterCardUserID []uint32 //自己创建的学生用户
	slaveCardUserID  []uint32 //被邀请查看的学生用户
}

/****************************************************************/

/***************************************************卡用户(学生、教师、职工)**/
var cardUserMap sync.Map //卡用户 CardUserID卡用户ID(uint32) 作为KEY   纯卡用户可以不绑定微信
type CardUserType struct {
	MasterWechatUnionID string
	SlaveWechatUnionID  []string
	AreaID              uint32
	Name                string
	Sex                 uint8
	// Birthday
	Grade                 uint8
	Class                 uint8
	StudentNo             string
	PhoneAreaServiceID    uint32
	SimCard               uint32
	WaterAreaServiceID    uint32
	WaterCard             uint32
	LocationAreaServiceID uint32
	LocationCard          uint32
	PosAreaServiceID      uint32
	PosCard               uint32
}

/*---------------------------------------------------------------*/

/*******************************************************电话设备**/
var phone20DeviceMap sync.Map //device_id 作为KEY  Phone20DeviceStruct 作为VALUE
type Phone20DeviceStruct struct {
	AreaServiceID uint32
	AllowCallTime uint8  //允许通话时长,255表示不限制时长
	Place         string //安装位置
	Status        bool
}

/*---------------------------------------------------------------*/

/********************************************************Sim卡信息**/
var simInfoMap sync.Map //card_id 作为KEY
type SimInfoStruct struct {
	CardUserID          uint32 //卡用户ID
	Status              int8   //0无数据   1 正常使用   2 服务已到期 3 已挂失
	MonthTalkMinutes    uint16
	MonthPackageMinutes uint16
	FamilyNum           []FamilyOnePerson // 如果一张sim卡，到多个学校绑定不一样的亲情号，则改成MAP，但是移动不允许。先这样。
}
type FamilyOnePerson struct {
	Phone    string
	Nickname string
}

/*---------------------------------------------------------------*/

///********************************************************Sim卡信息(允许一卡多校区使用)**/
//var simInfoMap sync.Map //card_id 作为KEY
//type SimInfoStruct struct {
//	Status              int8 //0没得这个数据   1 正常使用   其他各种异常
//	MonthTalkMinutes    uint16
//	MonthPackageMinutes uint16
//	AreaFamilyNum       []sync.Map //area_id为KEY , []familyOnePerson 为 VALUE
//}
//type FamilyOnePerson struct {
//	Phone    string
//	Nickname string
//}
///*---------------------------------------------------------------*/

///*******************************************************定位设备**/
//var phone20DeviceMap sync.Map //device_id 作为KEY  DeviceStruct 作为VALUE
//type Phone20DeviceStruct struct {
//	AreaID        int32
//	AllowCallTime uint8 //允许通话时长,255表示不限制时长
//	Status        bool
//}
///*---------------------------------------------------------------*/
//
//
///*******************************************************饮水设备**/
//var phone20DeviceMap sync.Map //device_id 作为KEY  DeviceStruct 作为VALUE
//type Phone20DeviceStruct struct {
//	AreaID        int32
//	AllowCallTime uint8 //允许通话时长,255表示不限制时长
//	Status        bool
//}
///*---------------------------------------------------------------*/

/***************************************************卡用户(学生、教师、职工)**/
var cardUserTrendMap sync.Map //卡用户 CardUserID卡用户ID(uint32) 作为KEY,[]CardUserTrendType作为VALUE
type CardUserTrendType struct {
	ServiceType uint8 //服务项目 UI图标及颜色等
	DeviceID    uint32
	Content     string
	Time        time.Time
}

/*---------------------------------------------------------------*/

func Init() {
	initOperatorMap()
	initWechatCustomerMap()
	initCardUserMap()
	initAreaServiceMap()
	initPhone20DeviceMap()
	initSimFamilyNumMap()
}
