package dbmodel

import (
	"database/sql"
	"dc_golang_server_1/logger"
	"go.uber.org/zap"
	"strings"
	"time"
)

//CREATE TABLE IF NOT EXISTS public.u_call_record
//(
//customer_id integer,
//customer_name character varying(64) COLLATE pg_catalog."default",
//customer_phone character varying(12) COLLATE pg_catalog."default",
//card_user_id integer,
//card_user_name character varying(32) COLLATE pg_catalog."default",
//card_code character varying(32) COLLATE pg_catalog."default",
//card_type smallint,
//called_number character varying(12) COLLATE pg_catalog."default",
//called_nickname character varying(32) COLLATE pg_catalog."default",
//call_start_at timestamp without time zone,
//talk_time smallint,
//area_id integer,
//operator_id integer,
//device_id bigint,
//telephone_package_id integer,
//state smallint,
//create_at timestamp without time zone DEFAULT CURRENT_TIMESTAMP
//)
type TableUserCallRecord struct {
	CustomerId     int32     //会员ID
	CustomerPhone  string    //会员手机号  //character varying(11) COLLATE pg_catalog."default",
	CardUserId     int32     //卡用户ID   //integer,
	CardUserName   string    //卡用户姓名  character varying(32) COLLATE pg_catalog."default",
	CardID         string    //卡ID     character varying(32) COLLATE pg_catalog."default",
	CardType       int8      //卡类型   smallint,
	CalledNumber   string    //被叫号码    character varying(12) COLLATE pg_catalog."default",
	CalledNickname string    //被叫昵称   character varying(32) COLLATE pg_catalog."default",
	CallStartAt    string    //time.Time //通话开始时间  without time zone, .Format("2006-01-02 15:04:05.000"),
	TalkTime       int16     //通话时长   //integer,
	AreaId         int32     //区域(校区)ID //integer,
	OperatorId     int32     //运营商ID     //integer,
	DeviceId       uint32    //设备ID      //bigint,
	Status         uint8     //通话状态    //smallint,
	CreateAt       time.Time // timestamp without time zone DEFAULT CURRENT_TIMESTAMP
}

func clearTransaction(tx *sql.Tx) {
	err := tx.Rollback()
	if err != sql.ErrTxDone && err != nil {
		logger.Log.Panic("InsertUCallRecordAndUpdateUSimInfo clearTransaction",
			zap.Error(err))
	}
}

func InsertUCallRecordAndUpdateUSimInfo(insertData TableUserCallRecord) uint8 { //0 成功 1 唯一约束重复 9 系统繁忙
	tx, err := PgDb.Begin()
	if err != nil {
		logger.Log.Error("InsertUCallRecordAndUpdateUSimInfo PgDb.Begin():",
			zap.Error(err))
		return 9
	}
	defer clearTransaction(tx)
	//先InsertUCallRecord
	_, err = tx.Exec("INSERT INTO u_call_record(customer_id,customer_phone,card_user_id,card_user_name,card_id,card_type,called_number,called_nickname,call_start_at,talk_time,area_id,operator_id,device_id,status)VALUES($1,$2,$3,$4,$5,$6,$7,$8,$9,$10,$11,$12,$13,$14)",
		insertData.CustomerId,
		insertData.CustomerPhone,
		insertData.CardUserId,
		insertData.CardUserName,
		insertData.CardID,
		insertData.CardType,
		insertData.CalledNumber,
		insertData.CalledNickname,
		insertData.CallStartAt,
		insertData.TalkTime,
		insertData.AreaId,
		insertData.OperatorId,
		insertData.DeviceId,
		insertData.Status,
	)
	if err != nil {
		if strings.Contains(err.Error(), "duplicate key value") { //todo 这里应考虑去查出来看看内容是不是一样的
			//fmt.Println("InsertUserCallRecord唯一约束重复", err)
			return 1
		}
		logger.Log.Error("InsertUCallRecordAndUpdateUSimInfo InsertUserCallRecord:",
			zap.Error(err))
		return 9
	}
	//rowAffected, err := rs.RowsAffected()
	//if err != nil {
	//	fmt.Println("插入电话记录表:rs.RowsAffected()", err)
	//	panic("反正是测试，先退出2")
	//}
	//fmt.Println(rowAffected)

	_, err = tx.Exec("UPDATE u_sim_info SET total_talk_minutes=total_talk_minutes+$1 WHERE card_id=$2", (insertData.TalkTime+59)/60, insertData.CardID)
	if err != nil {
		logger.Log.Error("InsertUCallRecordAndUpdateUSimInfo UPDATE u_sim_info:",
			zap.Error(err))
		return 9
	}
	//rowAffected, err = rs.RowsAffected()
	//if err != nil {
	//	fmt.Println("UPDATE u_sim_info:rs.RowsAffected()", err)
	//	panic("反正是测试，先退出4")
	//}
	//fmt.Println(rowAffected)

	if err = tx.Commit(); err != nil {
		logger.Log.Error("InsertUCallRecordAndUpdateUSimInfo Commit:",
			zap.Error(err))
		return 9
	}

	return 0
}
