package util

import (
	"encoding/base64"
	"errors"
)

func EncryptBytesToBase64(cleartext []byte, pswd string, iv string) (base64Result []byte) { //看base64库，返回切片更快，本来tcp发送也是切片,不过这里面string也是切片
	var pswdIv [16]byte
	var pswdSum byte
	var i int
	cleartextLen := len(cleartext)
	var result = make([]byte, cleartextLen+1)

	for i = 0; i < 16; i++ {
		pswdIv[i] = ((pswd[i] & 0xaa) | (pswd[15-i] & 0x55)) + iv[15-i] + byte(i)

		if i%2 == 0 {
			pswdSum += pswd[i] + iv[i]
		} else {
			pswdSum -= pswd[i] - iv[i]

		}
	}

	for i = 0; i < 16; i++ {
		if i%2 == 0 {
			pswdIv[i] = ^pswdIv[i] + pswdSum
		} else {
			pswdIv[i] = pswdIv[i] - ^pswdSum
		}
	}

	result[cleartextLen] = ^pswdSum
	for i = 0; i < cleartextLen; i++ {
		result[cleartextLen] += cleartext[i]
	}
	for i = 0; i < cleartextLen; i++ {

		result[i] = ^((cleartext[i] & 0x72) | (cleartext[cleartextLen-1-i] & 0x8d))
		if i%2 == 0 {
			result[i] += pswdIv[i%16] - result[cleartextLen]
		} else {
			result[i] -= pswdIv[i%16] + result[cleartextLen]
		}
	}

	base64Result = make([]byte, base64.StdEncoding.EncodedLen(cleartextLen+1)+2) //多出两个用来装起始结束符
	base64.StdEncoding.Encode(base64Result[1:], result)
	return
	//return base64.StdEncoding.EncodeToString(result)
}

// DecryptBase64ToBytes 解密并转换为bytes
func DecryptBase64ToBytes(ciphertext []byte, pswd string, iv string) ([]byte, error) { //看base64库，传入切片更快，本来tcp接收也是切片,不过这里面string也是切片
	var pswdIv [16]byte
	var pswdSum byte
	var i int

	cipherBytes := make([]byte, base64.StdEncoding.DecodedLen(len(ciphertext)))
	hexLen, err := base64.StdEncoding.Decode(cipherBytes, ciphertext)
	//cipherBytes, err := base64.StdEncoding.DecodeString(ciphertext)
	if err != nil {
		return nil, err
	}

	clearLen := hexLen - 1
	var resultBytes = make([]byte, clearLen)

	var resultBytesTemp = make([]byte, clearLen)

	for i = 0; i < 16; i++ {
		pswdIv[i] = ((pswd[i] & 0xaa) | (pswd[15-i] & 0x55)) + iv[15-i] + byte(i)

		if i%2 == 0 {
			pswdSum += pswd[i] + iv[i]
		} else {
			pswdSum -= pswd[i] - iv[i]
		}
	}

	for i = 0; i < 16; i++ {
		if i%2 == 0 {
			pswdIv[i] = ^pswdIv[i] + pswdSum
		} else {
			pswdIv[i] = pswdIv[i] - ^pswdSum
		}
	}

	for i = 0; i < clearLen; i++ {
		if i%2 == 0 {
			resultBytesTemp[i] = ^(cipherBytes[i] - (pswdIv[i%16] - cipherBytes[clearLen]))
		} else {
			resultBytesTemp[i] = ^(cipherBytes[i] + (pswdIv[i%16] + cipherBytes[clearLen]))
		}
	}

	var sum byte
	for i = 0; i < clearLen; i++ {
		resultBytes[i] = (resultBytesTemp[i] & 0x72) | (resultBytesTemp[clearLen-1-i] & 0x8d)
		sum += resultBytes[i]
	}
	sum += ^pswdSum

	if sum != cipherBytes[clearLen] {
		return nil, errors.New("密文校验错误")
	}

	return resultBytes, nil
}

func EncryptBytesToBase64URL(cleartext []byte, pswd string) string {
	var i int
	cleartextLen := len(cleartext)
	var result = make([]byte, cleartextLen+1)

	result[cleartextLen] = 0x11
	for i = 0; i < cleartextLen; i++ {
		if i%2 == 0 {
			result[cleartextLen] -= cleartext[i] | 0x23
		} else {
			result[cleartextLen] += cleartext[i] & 0x36
		}
		result[cleartextLen] += ^((cleartext[i] >> 3) | (cleartext[i] << 5))
	}
	for i = 0; i < cleartextLen; i++ {
		result[i] = ^((cleartext[i] & 0x72) | (cleartext[cleartextLen-1-i] & 0x8d))
		if i%2 == 0 {
			result[i] += pswd[i%16] - result[cleartextLen]
		} else {
			result[i] -= pswd[i%16] + result[cleartextLen]
		}
	}

	return base64.URLEncoding.EncodeToString(result)
}

func DecryptBase64URLToBytes(ciphertext string, pswd string) ([]byte, error) { //
	var i int

	hex, err := base64.URLEncoding.DecodeString(ciphertext)
	if err != nil {
		return nil, err
	}

	clearLen := len(hex) - 1
	var resultBytes = make([]byte, clearLen)

	var resultBytesTemp = make([]byte, clearLen)
	for i = 0; i < clearLen; i++ {
		if i%2 == 0 {
			resultBytesTemp[i] = ^(hex[i] - (pswd[i%16] - hex[clearLen]))
		} else {
			resultBytesTemp[i] = ^(hex[i] + (pswd[i%16] + hex[clearLen]))
		}
	}

	var sum byte
	for i = 0; i < clearLen; i++ {
		resultBytes[i] = (resultBytesTemp[i] & 0x72) | (resultBytesTemp[clearLen-1-i] & 0x8d)
		//sum += resultBytes[i]
		if i%2 == 0 {
			sum -= resultBytes[i] | 0x23
		} else {
			sum += resultBytes[i] & 0x36
		}
		sum += ^((resultBytes[i] >> 3) | (resultBytes[i] << 5))
	}
	sum += 0x11

	if sum != hex[clearLen] {
		return nil, errors.New("密文校验错误")
	}

	return resultBytes, nil
}
