package v1

import (
	"dc_golang_server_1/cache"
	"github.com/gin-gonic/gin"
	"net/http"
)

// GetProvinceList 获取省份列表
// @Tags 公用接口
// @Summary  获取全国省份列表
// @Description 获取带首字母的省份列表
// @Product json
// @Success 200 {object} []cache.District "按首字母排序好的列表,可直接用作展示"
// @Router /base/province [GET]
// @Security ApiKeyAuth
func GetProvinceList(c *gin.Context) {
	data := []cache.District{
		{1, "四川省", "S"},
		{1, "四川省", "S"},
	}
	c.JSON(http.StatusOK, gin.H{
		"status": 200,
		"data":   data,
	})
}

// GetCityList 获取城市列表
// @Tags 公用接口
// @Summary  通过省份ID获取城市列表
// @Description 获取带首字母的城市列表
// @Param provinceID path integer true "省份id"
// @Product json
// @Success 200 {object} []cache.District "按首字母排序好的列表,可直接用作展示"
// @Router /base/city/{provinceID} [GET]
// @Security ApiKeyAuth
func GetCityList(c *gin.Context) {
	//province, _ := strconv.Atoi(c.Param("provinceID"))
	data := make(map[string][]string)
	data["S"] = append(data["C"], "成都市")
	c.JSON(http.StatusOK, gin.H{
		"status": 200,
		"data":   data,
	})
}

// GetCountyList 获取区县列表
// @Tags 公用接口
// @Summary  通过城市ID获取区县列表
// @Description 获取带首字母的城市列表
// @Param cityID path integer true "城市ID"
// @Product json
// @Success 200 {object} []cache.District "按首字母排序好的列表,可直接用作展示"
// @Router /base/county/{cityID} [GET]
// @Security ApiKeyAuth
func GetCountyList(c *gin.Context) {
	//province, _ := strconv.Atoi(c.Param("cityID"))
	data := make(map[string][]string)
	data["S"] = append(data["W"], "温江区")
	c.JSON(http.StatusOK, gin.H{
		"status": 200,
		"data":   data,
	})
}
