package service

import (
	"bytes"
	"crypto"
	"crypto/aes"
	"crypto/cipher"
	cryptoRand "crypto/rand"
	"crypto/rsa"
	"crypto/sha256"
	"crypto/x509"
	"dc_golang_server_1/logger"
	"encoding/base64"
	"encoding/json"
	"encoding/pem"
	"errors"
	"fmt"
	"go.uber.org/zap"
	"io/ioutil"
	"math/rand"
	"net/http"
	"net/url"
	"os"
	"strings"
	"sync"
	"time"
)

const appId = "wxff0ab56070f7f6c4"                                 // 小程序或者公众号的appid
const mchId = "1607746840"                                         // 微信支付的商户id
const privateSerialNo = "629D0627491C0FB9513BAFB1D22EDB60A10F4ABE" // 私钥证书号
const aesKey = "TIOROVNsdkjvkdsfjklLljF3012001sd"                  // 微信支付aes key
const privateKey = "MIIEvgIBADANBgkqhkiG9w0BAQEFAASCBKgwggSkAgEAAoIBAQC7TaGNwUXSqTje" +
	"C2XclKbajbQ7JefJY1Ge1dJxZI3VPFaxMSI6+L+ba5eq8+jBD1cTFO1Pti7sI6Rp" +
	"2vUanJzPdJwLb42R7hIwzw5YgN3lZ8F/k8HZtzMQGeLhx5IeL2xVUU/y5IbRr1N5" +
	"078IqU4Gx7c8/QwzQaR+CnJ0edzn6burSkvy5G1CYzK9XkaqXBXFWMZrLAXLMuAi" +
	"71Apqvki5rOAxCCdecpggXnkqv4Ed5ONGeWPxPz1rUUZDo8HdLyznzLjVLMup/gK" +
	"h76kF3QW53HPRuKAzHiHMsi3FBEbrGFb9tkYds3Pg/7Eg6ZF8PnOvENO93zpcsjs" +
	"HeP5PCKLAgMBAAECggEATHoDF8UthDlSHd+bhLgxdJVdXLU6cdEat3boySHcg49y" +
	"ekDskBWW7pe8wrF6fLGNxR3LU7u4ZvSaohoj7+y4XkBnooe2zznctY8ldV+QuqVl" +
	"rXigMwwJTm6zY6q/75E/c4qS0UMcBSvIMrsRijDOyhfrqnxL6fXIeyo613IWj539" +
	"2gX7UH9LdlN/QPQ2UGWmW5WdoMtxD/q/yCxjHgwqzvL+O3RKPQt2WTXu9Muw02gX" +
	"iovtoAJrBrHPa0H1BpHpKDKzfXgpUW5nJL1758zWPe6bLMLfP2PgacnfitdPgILV" +
	"82PXghy6D6YjgqtcAq1IM8e6p70HOfhzGiMT8jrhMQKBgQDl3sdclJJ8oOlMqkdJ" +
	"/bb5jhMStaoQfUipSjqoCMBDbjbQrVRKWgYaXZIguBN/dGdOtYExDcWmsZjQgx/K" +
	"Z8dC8/H0rQxbgfp8HLR99ud7634U5cuZSguJMUe/kkeP5OLmtexGLb8NaPEWwjqv" +
	"9YaVaJE+yHIaokk5lqwfHsFGaQKBgQDQmCdPUdHltYJ0nd86jA9/B6h/P/pZYSCC" +
	"L7b9AHGzJAmt0jeFDA2RJG0G2pQNzn9tS1xbCfsFoCwVokBQOmTMdA1MI8iZGMHg" +
	"C4PItIcTTa57iqnxjNNc0ndtKwwj62gJOyv2mDX3NxxVOeh201rHuzpPmhG2DhvN" +
	"Uk4ESdcK0wKBgQCoLx1SQWoEQX8wlo1eN4G+iRKvZ7csJV4abrMH6o8xRtxYRdkc" +
	"JU290KRvx/6MDFDhp68Egv0P/3S9apLB9rg15mIrEMmv0iPA1we/+fxYEG7JP3B9" +
	"kU1rbWD/azlssJzLDGP43NOiLhvm+OkTgOKdQkqy0tmZXw+QUQtSIaBrsQKBgEDY" +
	"7G+WSiCVewbyBXHYFQEyykk1BIlAzrBqUQL6Xl6It5GYJrZv9s/GeTaGQTzBk4/Q" +
	"u9d5lNY84zeeh0vpu2hsM64V+cjOj4ctTTzaEpwHrFSv2QFE/HiA3avMlCuehWtJ" +
	"P/ObgjtmoKJGLGS0dCZfqFMS3KqyXCZKjgJ7SiUNAoGBAI4QlFzt2LVuAD8+L6u2" +
	"newATfnFZXISIqI7x256bsbmhuPtTVy3lGKskxsKfuL7ey75XA9pmLf3nb1LGL2T" +
	"t34nCiCpMp0b3PLcEPhPW1XKH+OozYBjMpPu9MHNR1ZPd6Na/Ey3sJIx2wf3oNlw" +
	"jouLfNPUPBm0JxmNmqukhHV8"

// 对消息的散列值进行数字签名
func signPKCS1v15(msg, privateKey []byte, hashType crypto.Hash) ([]byte, error) {
	block, _ := pem.Decode(privateKey)
	if block == nil {
		return nil, errors.New("private key decode error")
	}
	pri, err := x509.ParsePKCS8PrivateKey(block.Bytes)
	if err != nil {
		return nil, errors.New("parse private key error")
	}
	key, ok := pri.(*rsa.PrivateKey)
	if ok == false {
		return nil, errors.New("private key format error")
	}
	sign, err := rsa.SignPKCS1v15(cryptoRand.Reader, key, hashType, msg)
	if err != nil {
		return nil, errors.New("sign error")
	}
	return sign, nil
}

// base编码
func base64EncodeStr(src []byte) string {
	return base64.StdEncoding.EncodeToString(src)
}

// 生成: 时间戳 + 设置前缀 ＋ 随即字符串
const alphanum = "0123456789ABCDEFGHIJKLMNOPQRSTUVWXYZabcdefghijklmnopqrstuvwxyz"

func randomString(strlen int) string {
	result := make([]byte, strlen)
	for i := 0; i < strlen; i++ {
		result[i] = alphanum[rand.Intn(len(alphanum))]
	}
	return string(result)
}

func hash256(message string) []byte {
	h := sha256.New()
	h.Write([]byte(message))
	return h.Sum(nil)
}

func authorization(method string, paramMap map[string]interface{}, rawUrl string) (token string, err error) {
	var body string
	if len(paramMap) != 0 {
		paramJsonBytes, err := json.Marshal(paramMap)
		if err != nil {
			return token, err
		}
		body = string(paramJsonBytes)
	}
	urlPart, err := url.Parse(rawUrl)
	if err != nil {
		return token, err
	}
	canonicalUrl := urlPart.RequestURI()
	timestamp := time.Now().Unix()
	nonce := randomString(32)
	message := fmt.Sprintf("%s\n%s\n%d\n%s\n%s\n", method, canonicalUrl, timestamp, nonce, body)
	if err != nil {
		return token, err
	}
	if err != nil {
		return token, err
	}
	signBytes, err := signPKCS1v15(hash256(message), []byte(privateKey), crypto.SHA256)
	if err != nil {
		return token, err
	}
	sign := base64EncodeStr(signBytes)
	token = fmt.Sprintf("mchid=\"%s\",nonce_str=\"%s\",timestamp=\"%d\",serial_no=\"%s\",signature=\"%s\"",
		mchId, nonce, timestamp, privateSerialNo, sign)
	return token, nil
}

// 获取公钥
const publicKeyUrl = "https://web.mch.weixin.qq.com/v3/certificates"

type TokenResponse struct {
	Data []TokenResponseData `json:"data"`
}
type TokenResponseData struct {
	EffectiveTime      string             `json:"effective_time"`
	EncryptCertificate EncryptCertificate `json:"encrypt_certificate"`
	ExpireTime         string             `json:"expire_time"`
	SerialNo           string             `json:"serial_no"`
}
type EncryptCertificate struct {
	Algorithm      string `json:"algorithm"`
	AssociatedData string `json:"associated_data"`
	Ciphertext     string `json:"ciphertext"`
	Nonce          string `json:"nonce"`
}

var publicSyncMap sync.Map

// 获取公钥
func getPublicKey() (key string, err error) {
	var prepareTime int64 = 24 * 3600 * 3 // 证书提前三天过期旧证书，获取新证书
	nowTime := time.Now().Unix()
	// 读取公钥缓存数据
	cacheValueKey := fmt.Sprintf("app_id:%s:public_key:value", appId)
	cacheExpireTimeKey := fmt.Sprintf("app_id:%s:public_key:expire_time", appId)
	cacheValue, keyValueOk := publicSyncMap.Load(cacheValueKey)
	cacheExpireTime, expireTimeOk := publicSyncMap.Load(cacheExpireTimeKey)
	if keyValueOk && expireTimeOk {
		// 格式化时间
		local, _ := time.LoadLocation("Local")
		location, _ := time.ParseInLocation(time.RFC3339, cacheExpireTime.(string), local)
		// 判断是否过期，证书没有过期直接返回
		if location.Unix()-prepareTime > nowTime {
			return cacheValue.(string), nil
		}
	}
	token, err := authorization(http.MethodGet, nil, publicKeyUrl)
	if err != nil {
		return key, err
	}
	request, err := http.NewRequest(http.MethodGet, publicKeyUrl, nil)
	if err != nil {
		return key, err
	}
	request.Header.Add("Authorization", "WECHATPAY2-SHA256-RSA2048 "+token)
	request.Header.Add("User-Agent", "用户代理(https://zh.wikipedia.org/wiki/User_agent)")
	request.Header.Add("Content-type", "application/json;charset='utf-8'")
	request.Header.Add("Accept", "application/json")
	client := http.DefaultClient
	response, err := client.Do(request)
	if err != nil {
		return key, err
	}
	defer response.Body.Close()
	bodyBytes, err := ioutil.ReadAll(response.Body)
	if err != nil {
		return key, err
	}
	//fmt.Println(string(bodyBytes))
	var tokenResponse TokenResponse
	if err = json.Unmarshal(bodyBytes, &tokenResponse); err != nil {
		return key, err
	}
	for _, encryptCertificate := range tokenResponse.Data {
		// 格式化时间
		local, _ := time.LoadLocation("Local")
		location, err := time.ParseInLocation(time.RFC3339, encryptCertificate.ExpireTime, local)
		if err != nil {
			return key, err
		}
		// 判断是否过期，证书没有过期直接返回
		if location.Unix()-prepareTime > nowTime {
			decryptBytes, err := decryptGCM(aesKey, encryptCertificate.EncryptCertificate.Nonce, encryptCertificate.EncryptCertificate.Ciphertext,
				encryptCertificate.EncryptCertificate.AssociatedData)
			if err != nil {
				return key, err
			}
			key = string(decryptBytes)
			publicSyncMap.Store(cacheValueKey, key)
			publicSyncMap.Store(cacheExpireTimeKey, encryptCertificate.ExpireTime)
			return key, nil
		}
	}
	return key, errors.New("get public key error")
}

func decryptGCM(aesKey, nonceV, ciphertextV, additionalDataV string) ([]byte, error) {
	key := []byte(aesKey)
	nonce := []byte(nonceV)
	additionalData := []byte(additionalDataV)
	ciphertext, err := base64.StdEncoding.DecodeString(ciphertextV)
	if err != nil {
		return nil, err
	}
	block, err := aes.NewCipher(key)
	if err != nil {
		return nil, err
	}
	aesGCM, err := cipher.NewGCM(block)
	if err != nil {
		return nil, err
	}
	plaintext, err := aesGCM.Open(nil, nonce, ciphertext, additionalData)
	if err != nil {
		return nil, err
	}
	return plaintext, err
}

func jsApi(payResMap map[string]string) (payJson string, err error) {
	payMap := make(map[string]string)
	timeStamp := time.Now().Unix()
	nonce := randomString(32)
	packageStr := "prepay_id=" + payResMap["prepay_id"]
	payMap["appId"] = appId
	payMap["timeStamp"] = fmt.Sprintf("%v", timeStamp)
	payMap["nonceStr"] = nonce
	payMap["package"] = packageStr
	// 签名
	message := fmt.Sprintf("%s\n%s\n%s\n%s\n", appId, fmt.Sprintf("%v", timeStamp), nonce, packageStr)
	open, err := os.Open("/Users/apple/data/www/go/work/src/study/testwechantpay/private.pem")
	if err != nil {
		return payJson, err
	}
	defer open.Close()
	privateKey, err := ioutil.ReadAll(open)
	if err != nil {
		return payJson, err
	}
	signBytes, err := signPKCS1v15(hash256(message), privateKey, crypto.SHA256)
	if err != nil {
		return payJson, err
	}
	sign := base64EncodeStr(signBytes)
	payMap["signType"] = sign
	payMap["paySign"] = "RSA"
	payJsonBytes, err := json.Marshal(payMap)
	if err != nil {
		return payJson, err
	}
	payJson = string(payJsonBytes)
	return payJson, nil
}

// 统一下单接口
func IspTransactionsJsapi() (payResMap map[string]string, err error) {
	jsapiUrl := "https://web.mch.weixin.qq.com/v3/pay/partner/transactions/jsapi"
	payResMap = make(map[string]string)
	amount := 10
	paramMap := make(map[string]interface{})
	paramMap["appid"] = appId
	paramMap["mchid"] = mchId
	paramMap["description"] = fmt.Sprintf("充值:￥%d", amount)
	paramMap["out_trade_no"] = "202109300001"
	paramMap["notify_url"] = "http://tools.localhost/notify"
	paramMap["amount"] = map[string]interface{}{"total": amount * 100, "currency": "CNY"}
	paramMap["payer"] = map[string]string{"openid": "opCO05utXkPQh3Vje13WjEdQpAZ4"}
	jsonString, err := json.MarshalIndent(paramMap, "", " ")
	logger.Log.Debug("微信jsapi下单开始：" + string(jsonString))
	logger.Log.Debug("微信jsapi下单开始",
		zap.String("params", string(jsonString)))
	token, err := authorization(http.MethodPost, paramMap, jsapiUrl)
	if err != nil {
		return payResMap, err
	}
	marshal, _ := json.Marshal(paramMap)
	request, err := http.NewRequest(http.MethodPost, jsapiUrl, bytes.NewReader(marshal))
	if err != nil {
		return payResMap, err
	}
	request.Header.Add("Authorization", "WECHATPAY2-SHA256-RSA2048 "+token)
	request.Header.Add("User-Agent", "用户代理(https://zh.wikipedia.org/wiki/User_agent)")
	request.Header.Add("Content-type", "application/json;charset='utf-8'")
	request.Header.Add("Accept", "application/json")
	client := http.DefaultClient
	response, err := client.Do(request)
	if err != nil {
		return payResMap, err
	}
	defer func() {
		response.Body.Close()
	}()
	bodyBytes, err := ioutil.ReadAll(response.Body)
	logger.Log.Debug("微信jsapi下单结束：" + string(bodyBytes))
	if err != nil {
		return payResMap, err
	}
	if err = json.Unmarshal(bodyBytes, &payResMap); err != nil {
		return payResMap, err
	}
	if payResMap["prepay_id"] == "" {
		return payResMap, errors.New("code:" + payResMap["code"] + "err:" + payResMap["message"])
	}
	return payResMap, nil
}

type WxOAuth2ResponseData struct {
	Appid   string
	Openid  string
	Unionid string
	Errcode string
	Errmsg  string
}

type BaseWechatPayConfig struct {
	appId  string
	secret string
}

//获取微信小程序配置信息
func getBaseWechatPayConfig(appId string) (baseWechatPayConfig BaseWechatPayConfig) {
	baseWechatPayConfig.appId = appId
	baseWechatPayConfig.secret = "bde10ef69cd62c3394c7f86b637e0619"
	return baseWechatPayConfig
}

func JsCode2session(code string) (responseData WxOAuth2ResponseData, err error) {
	var NAME = "微信授权"
	var name = NAME + "-code2Session"
	var url = "https://web.weixin.qq.com/sns/jscode2session?appid=${appId}&secret=${secret}&js_code=${code}&grant_type=authorization_code"
	baseWechatPayConfig := getBaseWechatPayConfig(appId)
	url = strings.ReplaceAll(url, "${appId}", baseWechatPayConfig.appId)
	url = strings.ReplaceAll(url, "${secret}", baseWechatPayConfig.secret)
	url = strings.ReplaceAll(url, "${code}", code)
	logger.Log.Debug(name,
		zap.String("url", url))
	request, err := http.NewRequest(http.MethodGet, url, nil)
	client := http.DefaultClient
	response, err := client.Do(request)
	if err != nil {
		return responseData, err
	}
	defer func() {
		response.Body.Close()
	}()
	bodyBytes, err := ioutil.ReadAll(response.Body)
	logger.Log.Debug(name,
		zap.String("url", url),
		zap.String("response", string(bodyBytes)))
	if err != nil {
		return responseData, err
	}
	if err = json.Unmarshal(bodyBytes, &responseData); err != nil {
		return responseData, err
	}
	if responseData.Errmsg != "" {
		return responseData, errors.New(responseData.Errmsg)
	}
	return responseData, nil
}
