package web

import (
	"dc_golang_server_1/wechat/model"
	wechatService "dc_golang_server_1/wechat/service"
	"dc_golang_server_1/wechat/web/response"
	responseState "dc_golang_server_1/wechat/web/response/state"
	"github.com/gin-gonic/gin"
	"net/http"
)

type TestResponse struct {
	Code responseState.State `json:"code"` //1000为成功，其他都是失败
	Msg  string              `json:"msg"`  //code非1000时的错误描述
	Data model.UWeCustomer   `json:"data"`
}

// @Tags 微信
// @Summary  测试
// @Description 测试使用
// @Param unionid query string true "用户微信的unionid"
// @Product json
// @Success 200 {object} TestResponse"
// @Router /v1/test [GET]
func test(c *gin.Context) {
	unionid := c.Query("unionid")
	var customer model.UWeCustomer
	customer = wechatService.FindUWeCustomerByUnionid(unionid)
	if customer.Id == 0 {
		c.JSON(http.StatusOK, response.ServerErrorResponse{Code: responseState.StateBusinessError, Msg: "用户不存在"})
	} else {
		c.JSON(http.StatusOK, response.ServerDataResponse{Code: responseState.StateOK, Data: customer})
	}
}

// @Tags 微信
// @Summary  登录
// @Description 微信code传至后台,后台返回token,token有效时长:24小时
// @Param code path string true "用户微信的code"
// @Product json
// @Success 200 {object} service.LoginFromWechatAppletResponse"
// @Router /v1/login/{code} [POST]
func weLogin(c *gin.Context) {
	code := c.Param("code")
	responseData, err := wechatService.LoginFromWechatApplet(code)
	if err != (response.ServerErrorResponse{}) {
		c.JSON(http.StatusOK, err)
	} else {
		c.JSON(http.StatusOK, responseData)
	}
}
